import * as Cesium from 'cesium';

import { VIEWER_OPTIONS_FN } from './VIEWER_OPTIONS';

export interface GroundStationOptions {
  height?: number; // 可选高度，默认为0
  id: string; // 站点的唯一标识符
  latitude: number;
  longitude: number;
  name: string;
  pixelSize?: number; // 点的可选像素大小
}

export class HCesiumViewerCls {
  private viewer: Cesium.Viewer | null = null;
  // 用于存储当前地面站实体的 Map
  currentStationEntities: Map<string, Cesium.Entity> = new Map();

  /**
   * 初始化 Cesium Viewer
   * @param container - 用于承载 Cesium Viewer 的 DOM 元素或其 ID
   */
  initCesiumViewer(container: ConstructorParameters<typeof Cesium.Viewer>[0]) {
    this.viewer = new Cesium.Viewer(container, VIEWER_OPTIONS_FN());
    // 初始化时清空可能存在的旧实体引用
    this.currentStationEntities.clear();
  }

  /**
   * 向视图中添加地面站实体
   * @param options - 地面站的选项参数
   * @returns 添加的实体对象，如果添加失败则返回 null。
   */
  addGroundStation(options: GroundStationOptions): Cesium.Entity | null {
    if (!this.viewer) {
      console.error('视图未初始化。无法添加地面站。');
      return null;
    }

    // 检查是否已存在相同 ID 的实体
    if (this.currentStationEntities.has(options.id)) {
      console.warn(`ID 为 "${options.id}" 的地面站实体已存在，跳过添加。`);
      return this.currentStationEntities.get(options.id) || null;
    }

    // 解构赋值获取站点信息
    const { height = 0, id, latitude, longitude, name, pixelSize = 10 } = options;
    const position = Cesium.Cartesian3.fromDegrees(longitude, latitude, height);

    const groundStationEntity = new Cesium.Entity({
      // 使用传入的 id 作为实体的唯一标识符
      id: id,
      label: {
        font: '14pt sans-serif',
        outlineWidth: 2,
        pixelOffset: new Cesium.Cartesian2(0, -12), // 标签略微偏移到点的上方
        style: Cesium.LabelStyle.FILL_AND_OUTLINE,
        text: name,
        verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
      },
      name: name,
      point: {
        color: Cesium.Color.fromRandom(), // 随机颜色
        outlineColor: Cesium.Color.WHITE,
        outlineWidth: 2,
        pixelSize,
      },
      position: position,
    });

    const addedEntity = this.viewer.entities.add(groundStationEntity);
    // 添加成功后，将其存入 Map
    this.currentStationEntities.set(id, addedEntity);
    return addedEntity;
  }

  /**
   * 从视图中移除指定的地面站实体 (通过 ID)
   * @param entityId - 要移除的地面站实体的 ID
   * @returns 如果成功移除则返回 true，否则返回 false
   */
  removeGroundStationById(entityId: string): boolean {
    if (!this.viewer) {
      console.error('视图未初始化。无法移除地面站。');
      return false;
    }
    const entityToRemove = this.currentStationEntities.get(entityId);
    if (entityToRemove) {
      const removed = this.viewer.entities.remove(entityToRemove);
      if (removed) {
        // 移除成功后，从 Map 中删除
        this.currentStationEntities.delete(entityId);
        return true;
      } else {
        console.warn(`尝试从 Cesium 移除 ID 为 "${entityId}" 的实体失败。`);
        return false; // Cesium 移除失败
      }
    } else {
      // console.warn(`未在 Map 中找到 ID 为 "${entityId}" 的地面站实体，无法移除。`); // 可能在 clearAll 时触发，不一定是警告
      return false; // Map 中未找到
    }
  }

  clearAllGroundStations() {
    if (!this.viewer) return;
    for (const entity of this.currentStationEntities.values()) {
      this.viewer?.entities.remove(entity);
    }
    this.currentStationEntities.clear();
  }

  destroy() {
    if (this.viewer) {
      this.clearAllGroundStations();
      this.viewer.destroy();
      this.viewer = null;
    }
    this.currentStationEntities.clear(); // 确保清空
  }
}
